import React from "react";

import { TimerModel } from "./TimerModel";

const TimerContext = React.createContext({
  ms: 0,
  s: 0,
  m: 0,
  h: 0,
  d: 0,
  formatValue: (value) => String(value),
});

const TimerValue = ({ unit, formatValue }) => (
  <Timer.Consumer>
    {(props) => {
      const format = formatValue || props.formatValue;

      return format(props[unit]) || null;
    }}
  </Timer.Consumer>
);

const Milliseconds = (props) => <TimerValue unit="ms" {...props} />;

const Seconds = (props) => <TimerValue unit="s" {...props} />;

const Minutes = (props) => <TimerValue unit="m" {...props} />;

const Hours = (props) => <TimerValue unit="h" {...props} />;

const Days = (props) => <TimerValue unit="d" {...props} />;

class Timer extends React.PureComponent {
  static Consumer = TimerContext.Consumer;
  static Milliseconds = Milliseconds;
  static Seconds = Seconds;
  static Minutes = Minutes;
  static Hours = Hours;
  static Days = Days;

  static defaultProps = {
    timeToUpdate: 1000,
    direction: "forward",
    initialTime: 0,
    startImmediately: true,
    lastUnit: "d",
    checkpoints: [],
    children: null,
    formatValue: (value) => String(value),
    onStart: () => {},
    onResume: () => {},
    onPause: () => {},
    onStop: () => {},
    onReset: () => {},
  };

  static getUI(children, renderProps) {
    if (children === null) {
      return null;
    }

    if (Array.isArray(children) || React.isValidElement(children)) {
      return children;
    }

    if (children.prototype && children.prototype.isReactComponent) {
      return React.createElement(children, renderProps);
    }

    if (typeof children === "function") {
      return children(renderProps);
    }

    throw new Error("Please use one of the supported APIs for children");
  }

  constructor(props) {
    super(props);

    const { initialTime, direction, timeToUpdate, lastUnit, checkpoints } =
      this.props;

    this.timer = new TimerModel({
      initialTime,
      direction,
      timeToUpdate,
      lastUnit,
      checkpoints,
      onChange: this.setState.bind(this),
    });

    this.state = {
      ...this.timer.timeParts,
      timerState: "INITED",
    };

    this.start = this.start.bind(this);
    this.pause = this.pause.bind(this);
    this.resume = this.resume.bind(this);
    this.stop = this.stop.bind(this);
    this.reset = this.reset.bind(this);
    this.setTime = this.setTime.bind(this);
    this.getTime = this.getTime.bind(this);
    this.getTimerState = this.getTimerState.bind(this);
    this.setDirection = this.setDirection.bind(this);
    this.setCheckpoints = this.setCheckpoints.bind(this);
  }

  componentDidMount() {
    const { startImmediately } = this.props;

    startImmediately && this.timer.start();
  }

  componentWillUnmount() {
    this.timer.stop();
  }

  render() {
    const {
      start,
      pause,
      resume,
      stop,
      reset,
      getTime,
      getTimerState,
      setTime,
      setDirection,
      setCheckpoints,
    } = this;
    const { ms, s, m, h, d, timerState } = this.state;
    const { formatValue, children } = this.props;

    return (
      <TimerContext.Provider value={{ ms, s, m, h, d, formatValue }}>
        {Timer.getUI(children, {
          start,
          resume,
          pause,
          stop,
          reset,
          getTime,
          getTimerState,
          setTime,
          setDirection,
          setCheckpoints,
          timerState,
        })}
      </TimerContext.Provider>
    );
  }

  setTime(time) {
    this.timer.setTime(time);
  }

  getTime() {
    return this.timer.getTime();
  }

  getTimerState() {
    return this.timer.state;
  }

  setDirection(direction) {
    this.timer.setDirection(direction);
  }

  setCheckpoints(checkpoints) {
    this.timer.setCheckpoints(checkpoints);
  }

  start() {
    this.timer.start();

    this.props.onStart();
  }

  stop() {
    this.timer.stop();

    this.props.onStop();
  }

  pause() {
    this.timer.pause();

    this.props.onPause();
  }

  reset() {
    this.timer.reset();

    this.props.onReset();
  }

  resume() {
    this.timer.resume();

    this.props.onResume();
  }
}

export default Timer;
