// import getTimeParts from '../helpers/getTimeParts';
// import now from '../helpers/now';
import TimerState from './TimerState';

function getTimeParts(time, lastUnit) {
  const units = ['ms', 's', 'm', 'h', 'd'];
  const lastUnitIndex = units.findIndex(unit => unit === lastUnit);

  const dividers = [1000, 60, 60, 24, 1];
  const dividersAcc = [1, 1000, 60000, 3600000, 86400000];

  const startValue = {
    ms: 0,
    s: 0,
    m: 0,
    h: 0,
    d: 0,
  };

  const output = units.reduce(
    (obj, unit, index) => {
      if (index > lastUnitIndex) {
        obj[unit] = 0;
      } else if (index === lastUnitIndex) {
        obj[unit] = Math.floor(time / dividersAcc[index]);
      } else {
        obj[unit] = Math.floor(time / dividersAcc[index]) % dividers[index];
      }

      return obj;
    },
    startValue,
  );

  return output;
}

function now() {
  if (typeof window === 'undefined' || !('performance' in window)) {
    return Date.now();
  }

  return performance.now();
}


export class TimerModel {
  initialTime;
  internalTime;
  time;
  direction;
  timeToUpdate;
  lastUnit;
  checkpoints;
  innerState;
  onChange;
  timerId;

  constructor({
    initialTime,
    direction,
    timeToUpdate,
    lastUnit,
    checkpoints,
    onChange,
  }) {
    this.internalTime = now();
    this.initialTime = initialTime;
    this.time = initialTime;
    this.direction = direction;
    this.timeToUpdate = timeToUpdate;
    this.lastUnit = lastUnit;
    this.checkpoints = checkpoints;
    this.innerState = new TimerState(onChange);
    this.onChange = onChange;

    this.timerId = null;
  }

  get state() {
    return this.innerState.getState();
  }

  get timeParts() {
    return this.getTimeParts(this.computeTime());
  }

  getTimeParts(time) {
    return getTimeParts(time, this.lastUnit);
  }

  setTime(time) {
    this.internalTime = now();
    this.initialTime = time;
    this.time = this.initialTime;

    this.onChange(this.getTimeParts(this.time));
  }

  getTime() {
    return this.time;
  }

  setLastUnit(lastUnit) {
    if (this.innerState.isPlaying()) {
      this.pause();
      this.lastUnit = lastUnit;
      this.resume(true);
    } else {
      this.lastUnit = lastUnit;
    }
  }

  setTimeToUpdate(interval) {
    if (this.innerState.isPlaying()) {
      this.pause();
      this.timeToUpdate = interval;
      this.resume();
    } else {
      this.timeToUpdate = interval;
    }
  }

  setDirection(direction) {
    this.direction = direction;
  }

  setCheckpoints(checkpoints) {
    this.checkpoints = checkpoints;
  }

  start() {
    if (this.innerState.setPlaying()) {
      this.setTimerInterval(true);
    }
  }

  resume(callImmediately = false) {
    if (!this.innerState.isStopped() && this.innerState.setPlaying()) {
      this.setTimerInterval(callImmediately);
    }
  }

  pause() {
    if (this.innerState.setPaused()) {
      clearInterval(this.timerId);
    }
  }

  stop() {
    if (this.innerState.setStopped()) {
      clearInterval(this.timerId);
    }
  }

  reset() {
    this.time = this.initialTime;

    this.onChange(this.getTimeParts(this.time));
  }

  setTimerInterval(callImmediately = false) {
    if (this.timerId) {
      clearInterval(this.timerId);
    }

    this.internalTime = now();

    const repeatedFunc = () => {
      const oldTime = this.time;
      const updatedTime = this.computeTime();

      this.onChange(this.getTimeParts(updatedTime));

      this.checkpoints.map(({ time, callback }) => {
        const checkForForward = time > oldTime && time <= updatedTime;
        const checkForBackward = time < oldTime && time >= updatedTime;
        const checkIntersection = this.direction === 'backward' ?
          checkForBackward :
          checkForForward;

        if (checkIntersection) {
          callback();
        }
      });
    };

    callImmediately && this.onChange(this.getTimeParts(this.time));

    this.timerId = window.setInterval(repeatedFunc, this.timeToUpdate);
  }

  computeTime() {
    if (this.innerState.isPlaying()) {
      const currentInternalTime = now();
      const delta = Math.abs(currentInternalTime - this.internalTime);

      switch (this.direction) {
        case 'forward':
          this.time = this.time + delta;
          this.internalTime = currentInternalTime;

          return this.time;

        case 'backward': {
          this.time = this.time - delta;
          this.internalTime = currentInternalTime;

          if (this.time < 0) {
            this.stop();

            return 0;
          }

          return this.time;
        }

        default:
          return this.time;
      }
    }

    return this.time;
  }
}